@@ -16,10 +16,6 @@
  * to execute arbitrary code. As such, if all but one thread drops
  * privilege, privilege escalation is somewhat trivial.
  */
-
-/* as per "man sigaction" */
-#define _POSIX_C_SOURCE 200809L
-
 #include <pthread.h>
 #include <signal.h>
 #include <stdio.h>
@@ -60,6 +56,7 @@ static void handler(int signum, siginfo_t *info, void *ignore) {
 
     pthread_cond_signal(&cond);
     pthread_mutex_unlock(&mu);
+
 }
 
 /*
@@ -86,9 +83,6 @@ int main(int argc, char **argv) {
     cap_t caps = cap_init();
     struct sigaction sig_action;
 
-    pthread_mutex_init(&mu, NULL);
-    pthread_cond_init(&cond, NULL);
-
     printf("program starting\n");
     if (pthread_create(&peer, NULL, victim, NULL)) {
 	perror("unable to start the victim thread");
@@ -119,7 +113,7 @@ int main(int argc, char **argv) {
     cap_free(caps);
 
     printf("no privilege in main process thread: len:%ld, caps:\"%s\"\n",
-	   (long int) greatest_len, text);
+	   greatest_len, text);
     if (greatest_len != 1) {
 	printf("failed to lower privilege as expected\n");
 	exit(1);
@@ -148,14 +142,13 @@ int main(int argc, char **argv) {
      */
 
     printf("greatest privilege in main process thread: len:%ld, caps:\"%s\"\n",
-	   (long int) greatest_len, text);
+	   greatest_len, text);
 
     cap_free(text);
     if (greatest_len != 1) {
 	printf("exploit succeeded\n");
 	exit(1);
+    } else {
+	printf("exploit failed\n");
     }
-
-    printf("exploit failed\n");
-    exit(0);
 }
